#include "samples.h"

int sched, pages, wpp;
long mask1, mask2, pagesize;
unsigned long long cmcopy_time[1000], chcopy_time[1000];
char *buff;

unsigned long long time1, time2;

void hit_task() {
	int top = pages*pagesize, inc = pagesize/wpp;
	for(int i = 0; i < 1000; i++) {
		/* Make sure we're actually doing a set every time */
		memset(buff, 'a', pages*pagesize);

		/* Actually do the time reading */
		time1 = RDTSC();
		for(char *j = buff; j < (char *)(buff + top); j += inc)
			*j = 'z';
		time2 = RDTSC();

		chcopy_time[i] = time2 - time1;
	}
}

void miss_task() {
	int top = pages*pagesize, inc = pagesize/wpp;
	for(int i = 0; i < 1000; i++) {
		sched_setaffinity(0, sizeof(mask1), (cpu_set_t *) &mask1);

		/* Make sure we're actually doing a set every time */
		memset(buff, 'a', pages*pagesize);

		sched_setaffinity(0, sizeof(mask2), (cpu_set_t *) &mask2);

		/* Actually do the time reading */
		time1 = RDTSC();
		for(char *j = buff; j < (char *)(buff + top); j += inc)
			*j = 'z';
		time2 = RDTSC();

		cmcopy_time[i] = time2 - time1;
	}
}

int main(int argc, char *argv[])
{
	int cpu1, cpu2;
	struct stat_data ch, cm;

	if(prep_test())
		return 1;

	if(argc > 3) {
		wpp = atoi(argv[1]);
		cpu1 = atoi(argv[2]);
		cpu2 = atoi(argv[3]);
		mask1 = pow(2, cpu1);
		mask2 = pow(2, cpu2);
	} else {
		cout << "Run with preemption_benchmark [writes/page] [cpu1] [cpu2]" << endl;
		return 0;
	}

	cout << "Migration cache-invalidate benchmark" << endl;
	cout << "Testing with " << wpp << " writes per page on cpus " << cpu1 << ", " << cpu2 << endl;

	pages = 1;
	pagesize = sysconf(_SC_PAGE_SIZE);
	cout << "Pagesize is " << pagesize << " bytes" << endl;

	for(int i = 0; i < 7; i++) {
		buff = (char*)malloc(pagesize*pages);

		/* Get the hit and miss times */
		hit_task();
		miss_task();

		selection_sort(cmcopy_time, 1000);
		selection_sort(chcopy_time, 1000);
		cm = stats(cmcopy_time, 990);
		ch = stats(chcopy_time, 990);

		cout << "Migration, 1 task, " << pages << " pages:\t" << cm.avg - ch.avg << " ticks" << endl;
		cout << "    Cache-cold writes took " << stat_print(cm) << ", cache-hot writes took " << stat_print(ch) << endl;

		pages *= 2;
		free(buff);
	}

	return 0;
}

