#include <sys/mman.h>
#include <sys/types.h>
#include <sys/syscall.h>
#include <iostream>
#include <fstream>
#include <math.h>
#include <pthread.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <unistd.h>
#include <chronos/chronos.h>
#include <chronos/chronos_utils.h>

using namespace std;

bool verbose = false;
int sched, pages, wpp; 
volatile int flag;
long mask1 = 1, mask2 = 2, pagesize;
unsigned long long preempt_sum_time[2], cmcopy_sum_time[2], chcopy_time;
char *buff[2];
char *dummy_buff;

unsigned long long time1, time2;

void hit_task() {
	long long chcopy_sum_time = 0;

	char *buff = (char*)malloc(pagesize*pages);

	for(int j = 0; j < 1000; j++) {
		/* Make sure we're actually doing a set every time */
		for(int i = 0; i < pages*pagesize; i += pagesize/wpp)
			buff[i] = '\a';

		/* Actually do the time reading */
		time1 = RDTSC();
		for(int i = 0; i < pages*pagesize; i += pagesize/wpp)
			buff[i] = '\0';
		time2 = RDTSC();
		chcopy_sum_time += (time2 - time1);
	}

	chcopy_time = chcopy_sum_time/1000;
	free(buff);
}

void miss_task(int id) {
	unsigned long preempt_time, cmcopy_time;

	preempt_time = 0;

	while(flag == 0) {}

	time1 = RDTSC();
	for(int i = 0; i < pages*pagesize; i += pagesize/wpp)
		buff[id][i] = '\a';
	time2 = RDTSC();

	cmcopy_time = time2 - time1;
	preempt_time = cmcopy_time - chcopy_time;

	cmcopy_sum_time[id] += cmcopy_time;
	preempt_sum_time[id] += preempt_time;
}

void *task1(void *p) {
	sched_setaffinity(0, sizeof(mask1), (cpu_set_t *) &mask1);
	miss_task(0);
	pthread_exit(NULL);
}

void *task2(void *p) {
	sched_setaffinity(0, sizeof(mask2), (cpu_set_t *) &mask2);
	miss_task(1);
	pthread_exit(NULL);
}

int main(int argc, char *argv[])
{
	pthread_t t1, t2;
	int cpu1, cpu2, dummyval;
	struct sched_param param;
	pthread_attr_t attr;
	unsigned long long preempt1, preempt2, cmcopy1, cmcopy2;

	// Lock the memory space
	if(mlockall(MCL_CURRENT|MCL_FUTURE))
		return 1;

	sched = SCHED_RT_FIFO;

	if(argc > 3) {
		wpp = atoi(argv[1]);
		cpu1 = atoi(argv[2]);
		cpu2 = atoi(argv[3]);
		mask1 = pow(2, cpu1);
		mask2 = pow(2, cpu2);
	} else {
		printf("Run with preemption_benchmark [writes/page] [cpu1] [cpu2]\n");
		return 0;
	}

	printf("Preemption cold-cache benchmark\n");
	printf("Testing with %d writes per page on cpus %d, %d\n", wpp, cpu1, cpu2);

	sched_setaffinity(0, sizeof(mask1), (cpu_set_t *) &mask1);
	/* Give ourselves a prio of 90 */
	param.sched_priority = 95;
	pthread_setschedparam(pthread_self(), SCHED_FIFO, &param);
	pthread_attr_init(&attr);

	srand ( time(NULL) );

	pages = 1;
	pagesize = sysconf(_SC_PAGE_SIZE);
	printf("Pagesize is %ld bytes\n", pagesize);

	param.sched_priority = 99;
	pthread_attr_setschedparam(&attr, &param);
	dummy_buff = (char*)malloc(pagesize*1536);

	for(int i = 0; i < 7; i++) {
		preempt_sum_time[0] = 0;
		cmcopy_sum_time[0] = 0;
		chcopy_time = 0;

		/* Get the hit time */
		hit_task();

		buff[0] = (char*)malloc(pagesize*pages);
		buff[1] = (char*)malloc(pagesize*pages);

		flag = 1;
		for(int k = 0; k < 1000; k++) {
			/* Flush cache */
			dummyval = rand() % 256;
			memset(dummy_buff, dummyval, pagesize*1536);
			/* Actually measure */
			pthread_create(&t1, &attr, task1, NULL);
			pthread_join(t1, NULL);
		}

		preempt1 = preempt_sum_time[0]/1000;
		cmcopy1 = cmcopy_sum_time[0]/1000;
		printf("Preemption, 1 task, %d pages:\t%llu ticks\n", pages, preempt1);
		printf("    Cache-cold writes took %llu ticks, cache-hot writes took %lld ticks\n", cmcopy1, chcopy_time);

		preempt_sum_time[0] = 0;
		preempt_sum_time[1] = 0;
		cmcopy_sum_time[0] = 0;
		cmcopy_sum_time[1] = 0;

		for(int k = 0; k < 1000; k++) {
			/* Flush cache */
			dummyval = rand() % 256;
			memset(dummy_buff, dummyval, pagesize*1536);
			/* Actually measure */
			flag = 0;
			pthread_create(&t1, &attr, task1, NULL);
			pthread_create(&t2, &attr, task2, NULL);
			flag = 1;
			pthread_join(t1, NULL);
			pthread_join(t2, NULL);
		}

		preempt1 = preempt_sum_time[0]/1000;
		preempt2 = preempt_sum_time[1]/1000;
		cmcopy1 = cmcopy_sum_time[0]/1000;
		cmcopy2 = cmcopy_sum_time[1]/1000;
		printf("Preemption, 2 tasks, %d pages:\t%llu, %llu ticks\n", pages, preempt1, preempt2);
		printf("    Cache-cold writes took %llu ticks, cache-hot writes took %llu ticks\n", cmcopy1, chcopy_time);
		printf("    Cache-cold writes took %llu ticks, cache-hot writes took %llu ticks\n", cmcopy2, chcopy_time);

		pages *= 2;
		free(buff[0]);
		free(buff[1]);

	}

	free(dummy_buff);

	return 0;
}

