/***************************************************************************
 *   Copyright (C) 2009-2011 Virginia Tech Real-Time Systems Lab	   *
 *   Written by Matthew Dellinger					   *
 *   mdelling@vt.edu							   *
 *									   *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or	   *
 *   (at your option) any later version.				   *
 *									   *
 *   This program is distributed in the hope that it will be useful,	   *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of	   *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the	   *
 *   GNU General Public License for more details.			   *
 *									   *
 *   You should have received a copy of the GNU General Public License	   *
 *   along with this program; if not, write to the			   *
 *   Free Software Foundation, Inc.,					   *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.		   *
 ***************************************************************************/
// hi!
#ifndef SAMPLES_H
#define SAMPLES_H

#include <sys/mman.h>
#include <sys/syscall.h>
#include <sys/types.h>
#include <iostream>
#include <math.h>
#include <pthread.h>
#include <signal.h>
#include <stdlib.h>
#include <stdio.h>
#include <time.h>
#include <unistd.h>
#include <chronos/chronos.h>
#include <chronos/chronos_utils.h>

using namespace std;

#define stat_print(x) x.avg << "," << x.stdev << " ticks"
#define stat_print_long(x) x.avg << "," << x.stdev << "," << x.min << "," << x.max << " ticks"

struct stat_data {
	unsigned long long avg;
	unsigned long long stdev;
	unsigned long long min;
	unsigned long long max;
};

int prep_test() {
	struct sched_param param;
	unsigned long mask = 1;

	sched_setaffinity(0, sizeof(mask), (cpu_set_t *) &mask);
	param.sched_priority = 99;
	if(pthread_setschedparam(pthread_self(), SCHED_FIFO, &param)) {
		cout << "Unable to set the scheduler" << endl;
		return 1;
	}

	// Lock the memory space
	if(mlockall(MCL_CURRENT|MCL_FUTURE)) {
		cout << "Unable to mlockall" << endl;
		return 1;
	}

	return 0;
}

void swap(unsigned long long& x, unsigned long long& y) {
	unsigned long long temp = x;
	x = y;
	y = temp;
}

void selection_sort(unsigned long long arr[], int size) {
	int indexOfMin, pass, j;

	for (pass = 0; pass < size - 1; pass++) {
		indexOfMin = pass;

		for (j = pass + 1; j < size; j++)
			if (arr[j] < arr[indexOfMin])
				indexOfMin = j;

		swap(arr[pass], arr[indexOfMin]);
	}
}

struct stat_data stats(unsigned long long arr[], int size) {
	struct stat_data stats;

	stats.avg = 0;
	stats.stdev = 0;

	for(int i = 0; i < size; i++)
		stats.avg += arr[i];

	stats.avg /= size;

	for(int i = 0; i < size; i++)
		stats.stdev += pow((arr[i] - stats.avg), 2);

	stats.stdev = sqrt(stats.stdev);
	stats.stdev /= size;

	stats.min = arr[0];
	stats.max = arr[size - 1];

	return stats;
}
#endif

