#!/bin/bash

INSTALL_DIR=/usr/local/chronos

if [ $(/usr/bin/id -u) -ne 0 ]; then
        echo "Error: Please run this script with sudo"
	exit 2
fi

if [ ! -f "./sched_test_app.jar" ]; then
        echo "You have not compiled sched_test_app_java."
	echo "Run 'make' before 'make install'."
	exit 2
fi

if [ ! -f "/usr/lib/libjchronos.so" ]; then
	echo "You must install libjchronos before continuing. Exiting install."
	exit 2
fi

if [ -f "${INSTALL_DIR}/slope_java.conf" ]; then
        echo "System slope already calculated..."
	cat $INSTALL_DIR/slope_java.conf
else
	echo "Finding the system slope..."
	sudo make calc_slope
	cat $INSTALL_DIR/slope_java.conf
fi

echo "Installing sched_test_app_java and sched_test_app_pvm in ${INSTALL_DIR}"
cp sched_test_app.jar $INSTALL_DIR/sched_test_app.jar
cp sched_test_app_java $INSTALL_DIR/sched_test_app_java
cp sched_test_app_pvm $INSTALL_DIR/sched_test_app_pvm
cp sched_test_app_jamaica $INSTALL_DIR/sched_test_app_jamaica
cp img/chronos.png $INSTALL_DIR/chronos.png

if [ ! -f "/usr/local/bin/sched_test_app_java" ]; then
        ln -s $INSTALL_DIR/sched_test_app_java /usr/local/bin/sched_test_app_java
fi

if [ ! -f "/usr/local/bin/sched_test_app_pvm" ]; then
        ln -s $INSTALL_DIR/sched_test_app_pvm /usr/local/bin/sched_test_app_pvm
fi

if [ ! -f "/usr/local/bin/sched_test_app_jamaica" ]; then
        ln -s $INSTALL_DIR/sched_test_app_jamaica /usr/local/bin/sched_test_app_jamaica
fi

echo "Copying jrts pre-compilation and -initialization lists to $INSTALL_DIR"
cp jrts_compilation_list $INSTALL_DIR
cp jrts_preinit_list $INSTALL_DIR

echo "Installation Complete."
