package edu.vt.realtime.testing;

import edu.vt.realtime.*;

import java.io.File;
import java.io.FileOutputStream;

public class SlopeCalculator {

	private double execSlope;
	private double higher = 1000;
	private double lower = 0;

	private Timespec deadline;
	private Timespec endtime;

	private ChronosAborts aborts;

	public SlopeCalculator() {
		deadline = new Timespec();
		endtime = new Timespec();
		aborts = ChronosScheduler.initAborts();
	}

	public void findSlope() {
		//calculate it, then save it to the file, if applicable
		double slope = calculateSlope();
		if(!writeToFile(slope))
			System.out.println("Error saving slope to /usr/local/chronos/slope_java.conf: Are you root (sudo)?");
		else
			System.out.println("Successfully saved slope to /usr/local/chronos/slope_java.conf");
	}

	private void evalSlope(int increments) {
		if(increments < 1 || increments > 1000000)
			increments = 1;

		int burn_total = 1000000;
		int burn_each = burn_total / increments;
		int burn_final = burn_total % increments;

		int tid = ChronosUtil.gettid();

		deadline.getTime();

		deadline.setSeconds(deadline.getSeconds() + 1);

		for (int i = 0; i < increments; i++){
			if(aborts.wasAborted(tid)){
				break;
			}
			ChronosUtil.burnCpu(burn_each, execSlope);
		}
		if (burn_final > 0)
			ChronosUtil.burnCpu(burn_final, execSlope);

		endtime.getTime();

		if(endtime.greaterThan(deadline))
			higher = execSlope + (higher - execSlope)*2/3;
		else
			lower = execSlope - (execSlope - lower)*2/3;

		execSlope = (higher + lower)/2;
	}

	public double calculateSlope() {
		ChronosUtil.setAffinity(1);
		ChronosUtil.setLinuxScheduler(LinuxSchedulerType.SCHED_FIFO);
		ChronosUtil.setPriority(48);

		this.execSlope = (this.higher + this.lower)/2;

		System.out.println("Slope is "+execSlope+", higher: "+higher+", lower: "+lower);
		for(int i = 0; i < 50; i++) {
			evalSlope(1000000);
			System.out.println("Slope is "+execSlope+", higher: "+higher+", lower: "+lower);
		}

		return execSlope;
	}

	private boolean writeToFile(double slope) {
		try{
			File slopeFile = new File("/usr/local/chronos/slope_java.conf");

			if(!slopeFile.exists())
				slopeFile.createNewFile();

			FileOutputStream fos = new FileOutputStream(slopeFile);
			String output = ""+slope+"\n";
			fos.write(output.getBytes());
			fos.flush();
			fos.close();

		}catch(Exception e){
			e.printStackTrace();
			return false;
		}

		return true;
	}

	public static void main(String[] args) {
		SlopeCalculator sc = new SlopeCalculator();
		sc.findSlope();
	}
}
