package edu.vt.realtime.testing;

import edu.vt.realtime.ChronosSchedulerType;
import java.io.OutputStream;
import java.io.PrintStream;

import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Text;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Spinner;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.widgets.FileDialog;

public class TestAppGui {
	
	private String taskSetPath;
	private ChronosTester tester;
	private TesterOptions opt;
	private PrintStream textAreaStream;

	private Shell shell;
	private Group outputOptionsGroup;
	private Label OutputLabel = null;
	private Button verboseCheckBox = null;
	private Button excelCheckBox = null;
	private Button GnuPlotCheckBox = null;
	private Button LogCheckBox = null;
	private Label CPUAffinityLabel = null;
	private Combo cpuAffinityCombo = null;
	private Group outputGroup;
	private Text outputTextArea = null;
	private Group schedulingOptionsGroup;
	private Label SchedulingAlgorithmLabel = null;
	private Combo AlgorithmCombo = null;
	private Button abortIdleTasksCheckBox = null;
	private Button HUAHandlersCheckBox = null;
	private Button UsePICheckBox = null;
	private Button DeadlockPreventionCheckBox = null;
	private Label LockingLabel = null;
	private Button LockingCheckBox = null;
	private Button NestedLockingCheckBox = null;
	private Composite composite = null;
	private Spinner csLengthSpinner = null;
	private Label CSLengthLabel;
	private Label RunPropertiesLabel = null;
	private Composite composite1 = null;
	private Label executionTimeLabel;
	private Spinner executionTimeSpinner;
	private Label logo;
	private Label logoSpacer;
	private Image logoImage;
	private Label lockingSeparator;
	private Label runPropertiesSeparator;
	private Label cpuAffinitySeparator;
	private Composite cpuUsageComposite;
	private Label CPUusageLabel;
	private Spinner cpuUsageSpinner;
	private Composite endUsageComposite;
	private Label endUsageLabel;
	private Spinner endUsageSpinner;
	private Composite iterateByComposite;
	private Label lblIterateBy;
	private Spinner iterateBySpinner;
	private Button batchModeCheckBox;
	private Label taskSetSeparator;
	private Label lblTaskSet;
	private Composite taskSetInfoComposite;
	private Label taskSetName;
	private Label taskSetUsage;
	private Label taskSetTasks;
	private Label taskSetPeriod;
	private Label taskSetLocks;
	private Button btnLoadTaskSet;
	private Label taskSetSpacer;
	private Composite composite_1;
	private Button btnClearOutput;
	private Button btnRun;
	private Composite composite_2;
	
	public TestAppGui() {
		tester = new ChronosTester();
		opt = new TesterOptions();

		createShell(); /* (Do this before the next step, so we have created outputTextArea, and so it doesn't throw a null pointer error on us */
	}

	private void createStreams() {
		/* Create PrintStream objects for TesterOptions */
		this.textAreaStream = new PrintStream(new TextOutputStream(outputTextArea));
		opt.out = textAreaStream;
		opt.err = textAreaStream;
	}

	private class TextOutputStream extends OutputStream {
		private Text txt;
		public TextOutputStream(Text t) {
			super();
			this.txt = t;
		}
		public synchronized void write(int b) {
			//get a byte from the int
			byte[] bite = {(byte)(b & 0x000000FF)};
			this.txt.insert(new String(bite));
		}
	}

	/**
	 * This method initializes sShell
	 * @wbp.parser.entryPoint
	 */
	private void createShell() {
		Display display = new Display();
		shell = new Shell(display);
		shell.setText("Sched Test App");
		createGroup();
		createGroup1();
		createGroup2();
		shell.setSize(new Point(1024, 768));
		shell.open();

		//we have to initialize the output streams after the GUI is created, but before the event loop
		createStreams();
		
		while (!shell.isDisposed ()) {
			if (!display.readAndDispatch())
				display.sleep();
		}
		display.dispose();
	}

	/**
	 * This method initializes group	
	 *
	 */
	private void createGroup() {
		shell.setLayout(new GridLayout(3, false));
		outputOptionsGroup = new Group(shell, SWT.NONE);
		outputOptionsGroup.setLayoutData(new GridData(SWT.LEFT, SWT.TOP, false, true, 1, 1));
		GridLayout gl_outputOptionsGroup = new GridLayout();
		outputOptionsGroup.setLayout(gl_outputOptionsGroup);
		
		logo = new Label(outputOptionsGroup, SWT.NONE);
		try{
			logoImage = new Image(Display.getCurrent() ,"img/chronos.png");
			logo.setImage(logoImage);
		} catch(Exception e) {
			try{
				logoImage = new Image(Display.getCurrent() ,"/usr/local/chronos/chronos.png");
				logo.setImage(logoImage);
			} catch(Exception f) {
				logo.setVisible(false);
			}
		}
		
		logoSpacer = new Label(outputOptionsGroup, SWT.NONE);
		OutputLabel = new Label(outputOptionsGroup, SWT.NONE);
		OutputLabel.setText("Output Options");
		verboseCheckBox = new Button(outputOptionsGroup, SWT.CHECK);
		verboseCheckBox.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent arg0) {
				if (verboseCheckBox.getSelection()){
					excelCheckBox.setEnabled(false);
					GnuPlotCheckBox.setEnabled(false);
				}else {
					excelCheckBox.setEnabled(true);
					GnuPlotCheckBox.setEnabled(true);
				}
			}
		});
		verboseCheckBox.setText("Verbose Output");
		excelCheckBox = new Button(outputOptionsGroup, SWT.CHECK);
		excelCheckBox.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent arg0) {
				if (excelCheckBox.getSelection()){
					verboseCheckBox.setEnabled(false);
					GnuPlotCheckBox.setEnabled(false);
				}else{
					verboseCheckBox.setEnabled(true);
					GnuPlotCheckBox.setEnabled(true);
				}
			}
		});
		excelCheckBox.setText("Output for Excel");
		GnuPlotCheckBox = new Button(outputOptionsGroup, SWT.CHECK);
		GnuPlotCheckBox.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent arg0) {
				if (GnuPlotCheckBox.getSelection()){
					verboseCheckBox.setEnabled(false);
					excelCheckBox.setEnabled(false);
				}else{
					verboseCheckBox.setEnabled(true);
					excelCheckBox.setEnabled(true);
				}
			}
		});
		GnuPlotCheckBox.setText("Output for Gnuplot");
		LogCheckBox = new Button(outputOptionsGroup, SWT.CHECK);
		LogCheckBox.setText("Log to file");
		{
			cpuAffinitySeparator = new Label(outputOptionsGroup, SWT.SEPARATOR | SWT.HORIZONTAL | SWT.CENTER);
			GridData gd_label_1 = new GridData(SWT.CENTER, SWT.CENTER, true, false, 1, 1);
			gd_label_1.widthHint = 300;
			cpuAffinitySeparator.setLayoutData(gd_label_1);
		}
		{
			composite_2 = new Composite(outputOptionsGroup, SWT.NONE);
			composite_2.setLayout(new GridLayout(2, false));
			CPUAffinityLabel = new Label(composite_2, SWT.SHADOW_NONE);
			CPUAffinityLabel.setText("CPU Affinity");
			cpuAffinityCombo = new Combo(composite_2, SWT.READ_ONLY);
		}
		createCombo();
	}

	/**
	 * This method initializes combo	
	 *
	 */
	private void createCombo() {
		Runtime runtime = Runtime.getRuntime();
		int processors = runtime.availableProcessors();
		for(int i=0; i < processors; i++)
			cpuAffinityCombo.add(""+i);
		cpuAffinityCombo.select(0);
	}

	/**
	 * This method initializes group1	
	 *
	 */
	private void createGroup1() {
		outputGroup = new Group(shell, SWT.NONE);
		outputGroup.setText("Output");
		outputGroup.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
		outputGroup.setLayout(new FillLayout(SWT.HORIZONTAL));
		outputTextArea = new Text(outputGroup, SWT.READ_ONLY | SWT.WRAP | SWT.V_SCROLL | SWT.MULTI);
		outputTextArea.setEditable(false);
	}

	/**
	 * This method initializes group2	
	 *
	 */
	private void createGroup2() {
		schedulingOptionsGroup = new Group(shell, SWT.NONE);
		schedulingOptionsGroup.setLayoutData(new GridData(SWT.LEFT, SWT.TOP, false, true, 1, 1));
		schedulingOptionsGroup.setLayout(new GridLayout());
		SchedulingAlgorithmLabel = new Label(schedulingOptionsGroup, SWT.NONE);
		SchedulingAlgorithmLabel.setText("Scheduling Algorithm");
		createAlgorithmCombo();
		abortIdleTasksCheckBox = new Button(schedulingOptionsGroup, SWT.CHECK);
		abortIdleTasksCheckBox.setText("Abort tasks on idle");
		HUAHandlersCheckBox = new Button(schedulingOptionsGroup, SWT.CHECK);
		HUAHandlersCheckBox.setText("Use HUA handlers");
		UsePICheckBox = new Button(schedulingOptionsGroup, SWT.CHECK);
		UsePICheckBox.setText("Use PI");
		UsePICheckBox.setEnabled(false);
		DeadlockPreventionCheckBox = new Button(schedulingOptionsGroup, SWT.CHECK);
		DeadlockPreventionCheckBox.setText("Deadlock prevention");
		DeadlockPreventionCheckBox.setEnabled(false);
		{
			lockingSeparator = new Label(schedulingOptionsGroup, SWT.SEPARATOR | SWT.HORIZONTAL | SWT.CENTER);
			GridData gd_lockingSeparator = new GridData(SWT.CENTER, SWT.CENTER, true, false, 1, 1);
			gd_lockingSeparator.widthHint = 200;
			lockingSeparator.setLayoutData(gd_lockingSeparator);
		}
		LockingLabel = new Label(schedulingOptionsGroup, SWT.NONE);
		LockingLabel.setText("Locking");
		LockingCheckBox = new Button(schedulingOptionsGroup, SWT.CHECK);
		LockingCheckBox.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent arg0) {
				if (LockingCheckBox.getSelection()) {
					UsePICheckBox.setEnabled(true);
					DeadlockPreventionCheckBox.setEnabled(true);
					csLengthSpinner.setEnabled(true);
					NestedLockingCheckBox.setEnabled(true);
				} else {
					UsePICheckBox.setEnabled(false);
					DeadlockPreventionCheckBox.setEnabled(false);
					csLengthSpinner.setEnabled(false);
					NestedLockingCheckBox.setEnabled(false);
				}
			}
		});
		LockingCheckBox.setText("Enable locking");
		NestedLockingCheckBox = new Button(schedulingOptionsGroup, SWT.CHECK);
		NestedLockingCheckBox.setText("Nested locking");
		NestedLockingCheckBox.setEnabled(false);
		createComposite();
		{
			runPropertiesSeparator = new Label(schedulingOptionsGroup, SWT.SEPARATOR | SWT.HORIZONTAL | SWT.CENTER);
			GridData gd_runPropertiesSeparator = new GridData(SWT.CENTER, SWT.CENTER, true, false, 1, 1);
			gd_runPropertiesSeparator.widthHint = 200;
			runPropertiesSeparator.setLayoutData(gd_runPropertiesSeparator);
		}
		RunPropertiesLabel = new Label(schedulingOptionsGroup, SWT.NONE);
		RunPropertiesLabel.setText("Run Properties");
		createComposite1();
	}

	/**
	 * This method initializes AlgorithmCombo	
	 *
	 */
	private void createAlgorithmCombo() {
		AlgorithmCombo = new Combo(schedulingOptionsGroup, SWT.READ_ONLY);
		ChronosSchedulerType types[] = ChronosSchedulerType.values();
		for (int i=0; i < types.length; i++) {
			AlgorithmCombo.add(types[i].toString().substring(9), types[i].ordinal());
		}
		AlgorithmCombo.select(0);
	}

	/**
	 * This method initializes composite	
	 *
	 */
	private void createComposite() {
		composite = new Composite(schedulingOptionsGroup, SWT.NONE);
		composite.setLayout(new RowLayout());
		CSLengthLabel = new Label(composite, SWT.NONE);
		CSLengthLabel.setText("CS length (%)  ");
		csLengthSpinner = new Spinner(composite, SWT.NONE);
		csLengthSpinner.setDigits(0);
		csLengthSpinner.setIncrement(5);
		csLengthSpinner.setMaximum(100);
		csLengthSpinner.setMinimum(1);
		csLengthSpinner.setEnabled(false);
	}

	/**
	 * This method initializes composite1	
	 *
	 */
	private void createComposite1() {
		composite1 = new Composite(schedulingOptionsGroup, SWT.NONE);
		composite1.setLayout(new RowLayout());
		executionTimeLabel = new Label(composite1, SWT.NONE);
		executionTimeLabel.setText("Execution time");
		executionTimeSpinner = new Spinner(composite1, SWT.NONE);
		executionTimeSpinner.setDigits(0);
		executionTimeSpinner.setIncrement(5);
		executionTimeSpinner.setMaximum(3600);
		executionTimeSpinner.setMinimum(1);
		{
			cpuUsageComposite = new Composite(schedulingOptionsGroup, SWT.NONE);
			cpuUsageComposite.setLayout(new RowLayout());
			{
				CPUusageLabel = new Label(cpuUsageComposite, SWT.NONE);
				CPUusageLabel.setText("CPU usage (%)");
			}
			{
				cpuUsageSpinner = new Spinner(cpuUsageComposite, SWT.NONE);
				cpuUsageSpinner.setPageIncrement(1000);
				cpuUsageSpinner.setMaximum(10000);
				cpuUsageSpinner.setMinimum(1);
				cpuUsageSpinner.setIncrement(100);
				cpuUsageSpinner.setDigits(0);
			}
		}
		{
			endUsageComposite = new Composite(schedulingOptionsGroup, SWT.NONE);
			endUsageComposite.setLayout(new RowLayout());
			{
				endUsageLabel = new Label(endUsageComposite, SWT.NONE);
				endUsageLabel.setText("End usage (%)");
			}
			{
				endUsageSpinner = new Spinner(endUsageComposite, SWT.NONE);
				endUsageSpinner.setEnabled(false);
				endUsageSpinner.setPageIncrement(1000);
				endUsageSpinner.setMaximum(10000);
				endUsageSpinner.setMinimum(1);
				endUsageSpinner.setIncrement(100);
				endUsageSpinner.setDigits(0);
			}
		}
		{
			iterateByComposite = new Composite(schedulingOptionsGroup, SWT.NONE);
			iterateByComposite.setLayout(new RowLayout());
			{
				lblIterateBy = new Label(iterateByComposite, SWT.NONE);
				lblIterateBy.setText("Iterate by (%)");
			}
			{
				iterateBySpinner = new Spinner(iterateByComposite, SWT.NONE);
				iterateBySpinner.setEnabled(false);
				iterateBySpinner.setPageIncrement(1000);
				iterateBySpinner.setMinimum(1);
				iterateBySpinner.setIncrement(100);
				iterateBySpinner.setDigits(0);
			}
		}
		{
			batchModeCheckBox = new Button(schedulingOptionsGroup, SWT.CHECK);
			batchModeCheckBox.addSelectionListener(new SelectionAdapter() {
				@Override
				public void widgetSelected(SelectionEvent arg0) {
					if(batchModeCheckBox.getSelection()) {
						iterateBySpinner.setEnabled(true);
						endUsageSpinner.setEnabled(true);
						CPUusageLabel.setText("Begin usage (%)");
						cpuUsageComposite.pack();
					} else {
						iterateBySpinner.setEnabled(false);
						endUsageSpinner.setEnabled(false);
						CPUusageLabel.setText("CPU usage (%)");
						cpuUsageComposite.pack();
					}
				}
			});
			batchModeCheckBox.setText("Batch mode");
		}
		{
			taskSetSeparator = new Label(schedulingOptionsGroup, SWT.SEPARATOR | SWT.HORIZONTAL | SWT.CENTER);
			GridData gd_taskSetSeparator = new GridData(SWT.CENTER, SWT.CENTER, true, false, 1, 1);
			gd_taskSetSeparator.widthHint = 200;
			taskSetSeparator.setLayoutData(gd_taskSetSeparator);
		}
		{
			lblTaskSet = new Label(schedulingOptionsGroup, SWT.NONE);
			lblTaskSet.setText("Task Set Information:");
		}
		{
			taskSetInfoComposite = new Composite(schedulingOptionsGroup, SWT.NONE);
			GridData gd_taskSetInfoComposite = new GridData(SWT.CENTER, SWT.CENTER, true, false, 1, 1);
			gd_taskSetInfoComposite.widthHint = 200;
			taskSetInfoComposite.setLayoutData(gd_taskSetInfoComposite);
			GridLayout gl_taskSetInfoComposite = new GridLayout(2, false);
			taskSetInfoComposite.setLayout(gl_taskSetInfoComposite);
			{
				taskSetName = new Label(taskSetInfoComposite, SWT.NONE);
				taskSetName.setLayoutData(new GridData(SWT.LEFT, SWT.CENTER, true, false, 1, 1));
				taskSetName.setText("Name: ");
			}
			new Label(taskSetInfoComposite, SWT.NONE);
			{
				taskSetTasks = new Label(taskSetInfoComposite, SWT.NONE);
				taskSetTasks.setLayoutData(new GridData(SWT.LEFT, SWT.CENTER, true, false, 1, 1));
				taskSetTasks.setText("Tasks:");
			}
			{
				taskSetPeriod = new Label(taskSetInfoComposite, SWT.NONE);
				taskSetPeriod.setLayoutData(new GridData(SWT.LEFT, SWT.CENTER, true, false, 1, 1));
				taskSetPeriod.setText("TS Period:");
			}
			{
				taskSetUsage = new Label(taskSetInfoComposite, SWT.NONE);
				taskSetUsage.setLayoutData(new GridData(SWT.LEFT, SWT.CENTER, true, false, 1, 1));
				taskSetUsage.setText("Usage:");
			}
			{
				taskSetLocks = new Label(taskSetInfoComposite, SWT.NONE);
				taskSetLocks.setLayoutData(new GridData(SWT.LEFT, SWT.CENTER, true, false, 1, 1));
				taskSetLocks.setText("Locks: ");
			}
			{
				btnLoadTaskSet = new Button(taskSetInfoComposite, SWT.NONE);
				btnLoadTaskSet.addSelectionListener(new SelectionAdapter() {
					@Override
					public void widgetSelected(SelectionEvent arg0) {
						FileDialog dialog = new FileDialog (shell, SWT.SAVE);
						String [] filterNames = new String [] {"All Files (*)"};
						String [] filterExtensions = new String [] {"*"};
						dialog.setFilterNames (filterNames);
						dialog.setFilterExtensions (filterExtensions);
						String filterPath = "/";
						try{ filterPath = taskSetPath.substring(0,taskSetPath.lastIndexOf("/")); } catch (Exception e) { }
						dialog.setFilterPath(filterPath);
						dialog.setFileName("");
						taskSetPath = dialog.open();
						
						loadTaskSet(taskSetPath);
					}
				});
				btnLoadTaskSet.setText("Load Task Set");
			}
			new Label(taskSetInfoComposite, SWT.NONE);
		}
		{
			taskSetSpacer = new Label(schedulingOptionsGroup, SWT.NONE);
			taskSetSpacer.setLayoutData(new GridData(SWT.CENTER, SWT.CENTER, false, true, 1, 1));
		}
		{
			composite_1 = new Composite(schedulingOptionsGroup, SWT.NONE);
			composite_1.setLayoutData(new GridData(SWT.CENTER, SWT.CENTER, true, false, 1, 1));
			composite_1.setLayout(new GridLayout(2, false));
			{
				btnClearOutput = new Button(composite_1, SWT.NONE);
				GridData gd_btnClearOutput = new GridData(SWT.LEFT, SWT.CENTER, true, false, 1, 1);
				gd_btnClearOutput.widthHint = 60;
				btnClearOutput.setLayoutData(gd_btnClearOutput);
				btnClearOutput.addSelectionListener(new SelectionAdapter() {
					@Override
					public void widgetSelected(SelectionEvent arg0) {
						outputTextArea.setText("");
					}
				});
				btnClearOutput.setText("Clear");
			}
			{
				btnRun = new Button(composite_1, SWT.NONE);
				GridData gd_btnRun = new GridData(SWT.LEFT, SWT.CENTER, true, false, 1, 1);
				gd_btnRun.widthHint = 60;
				btnRun.setLayoutData(gd_btnRun);
				btnRun.addSelectionListener(new SelectionAdapter() {
					@Override
					public void widgetSelected(SelectionEvent arg0) {
						runTests();
					}
				});
				btnRun.setText("Run");
			}
		}
	}

	private void disableGui() {
		this.btnRun.setEnabled(false);
		this.btnClearOutput.setEnabled(false);
		this.btnLoadTaskSet.setEnabled(false);
	}

	private void enableGui() {
		this.btnRun.setEnabled(true);
		this.btnClearOutput.setEnabled(true);
		this.btnLoadTaskSet.setEnabled(true);
	}

	/* End generated methods */
	/* The following are necessary for the GUI to interface with */

	private void println(String str) { print(str+"\n"); }
	private void print(String str) { outputTextArea.insert(str); }

	private synchronized void runTests() {
		disableGui();

		//get the options from the GUI
		guiToOptions();

		println("Running task set...\n");
		tester.prepareForRun(opt); //prepare the tester object for the run
		tester.run(); //and, actually run it
		println("\nFinished run.\n");

		enableGui();
	}

	private synchronized void loadTaskSet(String path) {
		disableGui();

		if (path == null || path.equals("")) {
			println("Error: No task set file selected.");
			return;
		}

		guiToOptions();
		opt.taskSetPath = path;

		/* Replace the PrintStream objects in the options, so loading this won't print anything */
		opt.out = System.out;
		opt.err = System.err;

		println("Loading task set from "+taskSetPath+ " ...");
		int hyperPeriod = tester.prepareForRun(opt); //and, prepare the tester object for the run

		/* Replace PrintStream objects in opt */
		opt.out = textAreaStream;
		opt.err = textAreaStream;

		if(hyperPeriod == -1) {
			println("Error: Task set file was invalid. Failed to load.");
			return;
		}

		/* Update the GUI to represent this new task set */
		String[] pathBroken = taskSetPath.split("/");
		String name = pathBroken[pathBroken.length-1];
		taskSetName.setText("Name: "+name);
		taskSetTasks.setText("Tasks: "+tester.getNumTasks());
		taskSetPeriod.setText("TS Period: "+hyperPeriod);
		String usageString = ""+tester.getUtilization();
		int idx = usageString.indexOf(".")+4;
		if (idx < usageString.length() && idx > 0)
				usageString = usageString.substring(0,idx);
		taskSetUsage.setText("Usage: "+usageString);
		taskSetLocks.setText("Locks: "+opt.lock_count);
		taskSetInfoComposite.pack();
		

		optionsToGui(); //make any changes in the options reflect in the GUI

		println("Task set loaded\n");

		enableGui();
	}

	private void guiToOptions() {
		//first, do the output options
		opt.verbose = verboseCheckBox.isEnabled() && verboseCheckBox.getSelection();
		opt.excel = excelCheckBox.isEnabled() && excelCheckBox.getSelection();
		opt.gnuplot = GnuPlotCheckBox.isEnabled() && GnuPlotCheckBox.getSelection();
		opt.log_to_file = LogCheckBox.isEnabled() && LogCheckBox.getSelection();

		//cpu affinity
		opt.main_cpu = Integer.parseInt(cpuAffinityCombo.getText());

		//scheduling
		opt.scheduler = ChronosSchedulerType.fromString(AlgorithmCombo.getText());
		opt.abort_idle = abortIdleTasksCheckBox.isEnabled() && abortIdleTasksCheckBox.getSelection();
		opt.use_hua = HUAHandlersCheckBox.isEnabled() && HUAHandlersCheckBox.getSelection();
		opt.use_pi = UsePICheckBox.isEnabled() && UsePICheckBox.getSelection();
		opt.deadlock_prevention = DeadlockPreventionCheckBox.isEnabled() && DeadlockPreventionCheckBox.getSelection();

		//locking
		opt.locking = LockingCheckBox.isEnabled() && LockingCheckBox.getSelection();
		opt.nested_locking = NestedLockingCheckBox.isEnabled() && NestedLockingCheckBox.getSelection(); //TODO - should there be a nested locking check box?
		opt.cs_len = csLengthSpinner.isEnabled()?csLengthSpinner.getSelection():0;

		//run properties
		opt.run_time = executionTimeSpinner.isEnabled()?executionTimeSpinner.getSelection():0;
		opt.cpu_usage = cpuUsageSpinner.isEnabled()?cpuUsageSpinner.getSelection():0;
		opt.cpu_usage_end = endUsageSpinner.isEnabled()?endUsageSpinner.getSelection():0;
		opt.cpu_usage_it = iterateBySpinner.isEnabled()?iterateBySpinner.getSelection():0;

		opt.batch_mode = batchModeCheckBox.isEnabled() && batchModeCheckBox.getSelection();
	}

	private void optionsToGui() {
		//first, do the output options
		verboseCheckBox.setSelection(opt.verbose);
		excelCheckBox.setSelection(opt.excel);
		GnuPlotCheckBox.setSelection(opt.gnuplot);
		if(opt.verbose) {
			verboseCheckBox.setEnabled(true);
			excelCheckBox.setEnabled(false);
			GnuPlotCheckBox.setEnabled(false);
		} else if (opt.excel) {
			excelCheckBox.setEnabled(true);
			verboseCheckBox.setEnabled(false);
			GnuPlotCheckBox.setEnabled(false);
		} else if(opt.gnuplot){
			GnuPlotCheckBox.setEnabled(true);
			excelCheckBox.setEnabled(false);
			verboseCheckBox.setEnabled(false);
		}
		LogCheckBox.setSelection(opt.log_to_file);

		//cpu affinity
		cpuAffinityCombo.setText(""+opt.main_cpu);

		//scheduling
		AlgorithmCombo.setText(opt.scheduler.toString().substring(9));
		abortIdleTasksCheckBox.setSelection(opt.abort_idle);
		HUAHandlersCheckBox.setSelection(opt.use_hua);
		UsePICheckBox.setSelection(opt.use_pi);
		DeadlockPreventionCheckBox.setSelection(opt.deadlock_prevention);

		//locking
		LockingCheckBox.setSelection(opt.locking);
		if(opt.locking) {
			LockingCheckBox.setEnabled(true);

			NestedLockingCheckBox.setEnabled(true);
			DeadlockPreventionCheckBox.setEnabled(true);
			UsePICheckBox.setEnabled(true);
			csLengthSpinner.setEnabled(true);
		} else {
			NestedLockingCheckBox.setEnabled(false);
			DeadlockPreventionCheckBox.setEnabled(false);
			UsePICheckBox.setEnabled(false);
			csLengthSpinner.setEnabled(false);
		}
		NestedLockingCheckBox.setSelection(opt.nested_locking);
		csLengthSpinner.setSelection(opt.cs_len);

		//run properties
		executionTimeSpinner.setSelection(opt.run_time);
		cpuUsageSpinner.setSelection(opt.cpu_usage);
		endUsageSpinner.setSelection(opt.cpu_usage_end);
		iterateBySpinner.setSelection(opt.cpu_usage_it);

		batchModeCheckBox.setSelection(opt.batch_mode);
		if(opt.batch_mode) {
			iterateBySpinner.setEnabled(true);
			endUsageSpinner.setEnabled(true);
			CPUusageLabel.setText("Begin usage (%)");
			cpuUsageComposite.pack();
		} else {
			iterateBySpinner.setEnabled(false);
			endUsageSpinner.setEnabled(false);
			CPUusageLabel.setText("CPU usage (%)");
			cpuUsageComposite.pack();
		}
	}
}
