#include "test.h"

unsigned long mask = 1;
chronos_mutex_t *r1;
chronos_mutex_t *r2;
chronos_aborts_t aborts;

struct timespec period;

/* This function should handle releasing every resource defined above */
void abort_task(int thread_id)
{
	THREAD_OUTPUT("thread %d - aborted\n", thread_id);
	if(!chronos_mutex_unlock(r1)) {
		THREAD_OUTPUT("thread %d - released 1\n", thread_id);
		test_val += 19;
	}
	if(!chronos_mutex_unlock(r2)) {
		THREAD_OUTPUT("thread %d - released 2\n", thread_id);
		test_val *= 21;
	}
	end_rtseg_self(END_PRIO);

	pthread_exit(NULL);
}

void *task1(void *p)
{
	int thread_id = gettid();

	sched_setaffinity(0, sizeof(mask), (cpu_set_t *) &mask);

	THREAD_OUTPUT("thread %d - start \n", thread_id);
	period.tv_sec = 2;
	begin_rtseg_self(RUN_PRIO, 0, NULL, &period, 0);

	test_val *= 7;

	//actual task block
	THREAD_OUTPUT("thread %d - locking lock 1\n", thread_id);
	if(chronos_mutex_lock(r1))
		abort_task(thread_id);
	THREAD_OUTPUT("thread %d - locked lock 1\n", thread_id);
	THREAD_OUTPUT("thread %d - burning cpu\n", thread_id);
	test_val -= 9;
	burn_cpu(1500000, exec_slope);
	THREAD_OUTPUT("thread %d - locking lock 2\n", thread_id);
	if(chronos_mutex_lock(r2))
		abort_task(thread_id);
	THREAD_OUTPUT("thread %d - locked lock 2\n", thread_id);	
	THREAD_OUTPUT("thread %d - unlocking lock 2\n", thread_id);
	test_val *= 11;
	chronos_mutex_unlock(r2);
	THREAD_OUTPUT("thread %d - unlocking lock 1\n", thread_id);
	chronos_mutex_unlock(r1);
	end_rtseg_self(END_PRIO);
	THREAD_OUTPUT("thread %d - end\n", thread_id);
	pthread_exit(NULL);
}

void *task2(void *p)
{
	int thread_id = gettid();

	sched_setaffinity(0, sizeof(mask), (cpu_set_t *) &mask);

	THREAD_OUTPUT("thread %d - start\n", thread_id);
	period.tv_sec = 1;
	begin_rtseg_self(RUN_PRIO, 0, NULL, &period, 0);

	test_val *= 13;

	//actual task block
	THREAD_OUTPUT("thread %d - locking lock 2\n", thread_id);
	if(chronos_mutex_lock(r2))
		abort_task(thread_id);
	THREAD_OUTPUT("thread %d - locked lock 2\n", thread_id);
	THREAD_OUTPUT("thread %d - locking lock 1\n", thread_id);
	test_val -= 15;
	if(chronos_mutex_lock(r1))
		abort_task(thread_id);
	THREAD_OUTPUT("thread %d - locked lock 1\n", thread_id);
	THREAD_OUTPUT("thread %d - unlocking lock 1\n", thread_id);
	test_val *= 17;
	chronos_mutex_unlock(r1);
	THREAD_OUTPUT("thread %d - unlocking lock 2\n", thread_id);
	chronos_mutex_unlock(r2);
	end_rtseg_self(END_PRIO);
	THREAD_OUTPUT("thread %d - end\n", thread_id);
	pthread_exit(NULL);
}

int main(int argc, char* argv[])
{
	pthread_t t1, t2;
	char c, optstring[]= "vh";

	/* Establish the signal handler. */
	if(init_aborts(&aborts)) {
		cout << "Error: Unable to mmap abort device" << endl;
		cout << "Are you running as root?" << endl;
	}

	r1 = new chronos_mutex_t();
	r2 = new chronos_mutex_t();
	chronos_mutex_init(r1);
	chronos_mutex_init(r2);
	sched = SCHED_RT_RMA | SCHED_FLAG_PI | SCHED_FLAG_NO_DEADLOCKS;

	while ((c = getopt (argc, argv, optstring)) != -1)
	{
		switch (c) {
		case 'v' :
			verbose = true;
			break;
		case 'h' :
			cout << "A simple test for deadlock resolution. Use -v for detailed output" << endl;
			return 0;
		}
	}

	exec_slope = get_burn_cpu_slope();
	if(exec_slope == 0) {
		cout << "Error: No slope found" << endl;
		return 0;
	}

	period.tv_sec = 0;
	period.tv_nsec = 0;

	if(prep_test())
		return 1;

	if(set_scheduler(sched, -1, mask)) {
		cout << "Error: Unable to set scheduler" << endl;
		cout << "Are you running as root?" << endl;
	}

	test_val +=5;

	pthread_create(&t1, NULL, task1, NULL);
	sleep(1);
	pthread_create(&t2, NULL, task2, NULL);
	sleep(1);
	pthread_join(t1, NULL);
	pthread_join(t2, NULL);

	if(test_val == 137676)
		cout << "Deadlock test successful" << endl;
	else
		cout << "Deadlock test failed: " << test_val << endl;

	chronos_mutex_destroy(r1);
	chronos_mutex_destroy(r2);
	delete r1;
	delete r2;
	return 0;
}

