#include "test.h"

unsigned long mask2 = 2;
unsigned long mask3 = 3;
chronos_mutex_t *r1;

struct timespec deadline;

void *task1(void *p)
{
	int thread_id = gettid(), result;
	sched_setaffinity(0, sizeof(mask3), (cpu_set_t *) &mask3);
	THREAD_OUTPUT("thread %d - start\n", thread_id);

	clock_gettime(CLOCK_REALTIME, &deadline);
	deadline.tv_sec += 15;
	begin_rtseg_self(RUN_PRIO, 10, &deadline, NULL, 10000000);

	test_val *= 7; //56, 0.0 seocnds

	//actual task block - 1.5 + 3 seconds
	THREAD_OUTPUT("thread %d - burning cpu unlocked\n", thread_id);
	burn_cpu(1500000, exec_slope);
	THREAD_OUTPUT("thread %d - locking lock 1\n", thread_id);
	result = chronos_mutex_lock(r1);
	if(result == 0)
		test_val -=13; //504, 1.5 seconds
	THREAD_OUTPUT("thread %d - locked lock 1\n", thread_id);
	THREAD_OUTPUT("thread %d - burning cpu locked\n", thread_id);
	burn_cpu(3500000, exec_slope);
	THREAD_OUTPUT("thread %d - unlocking lock 1\n", thread_id);

	test_val -= 25; //3295783, 4.0 seconds

	chronos_mutex_unlock(r1);
	burn_cpu(500000, exec_slope);
	end_rtseg_self(END_PRIO);
	THREAD_OUTPUT("thread %d - end\n", thread_id);

	test_val -= 5; //88986104, 6.0 seconds

	pthread_exit(NULL);
}

void *task2(void *p)
{
	int thread_id = gettid(), result;
	sched_setaffinity(0, sizeof(mask3), (cpu_set_t *) &mask3);
	THREAD_OUTPUT("thread %d - start\n", thread_id);

	test_val -= 9; //47, 1.0 seconds

	clock_gettime(CLOCK_REALTIME, &deadline);
	deadline.tv_sec += 15;
	begin_rtseg_self(RUN_PRIO, 10000, &deadline, NULL, 10000000);
	THREAD_OUTPUT("thread %d - locking lock 1\n", thread_id);
	result = chronos_mutex_lock(r1);
	if(result == 0)
		test_val *= 11; //517, 1.0 seconds
	THREAD_OUTPUT("thread %d - locked lock 1\n", thread_id);
	chronos_mutex_unlock(r1);
	THREAD_OUTPUT("thread %d - unlocking lock 1\n", thread_id);
	//actual task block - 3.5 seconds
	THREAD_OUTPUT("thread %d - burning cpu\n", thread_id);
	burn_cpu(2500000, exec_slope);
	end_rtseg_self(END_PRIO);
	THREAD_OUTPUT("thread %d - end\n", thread_id);

	test_val -= 21; //143296, 3.5 seconds

	pthread_exit(NULL);
}

void *task3(void *p)
{
	int thread_id = gettid(), result;
	sched_setaffinity(0, sizeof(mask3), (cpu_set_t *) &mask3);
	THREAD_OUTPUT("thread %d - start\n", thread_id);

	test_val *= 15; //7560, 2.0 seconds

	clock_gettime(CLOCK_REALTIME, &deadline);
	deadline.tv_sec += 15;
	begin_rtseg_self(RUN_PRIO, 1000, &deadline, NULL, 10000000);

	//actual task block - 0.5 + 2 seconds
	THREAD_OUTPUT("thread %d - burning cpu unlocked\n", thread_id);
	burn_cpu(500000, exec_slope);

	test_val -= 17; //7543, 2.5 seconds

	THREAD_OUTPUT("thread %d - locking lock 1\n", thread_id);
	result = chronos_mutex_lock(r1);
	THREAD_OUTPUT("thread %d - locked lock 1\n", thread_id);
	THREAD_OUTPUT("thread %d - burning cpu locked\n", thread_id);

	test_val *= 27; //88986141, 4.0 seconds

	burn_cpu(1000000, exec_slope);
	THREAD_OUTPUT("thread %d - unlocking lock 1\n", thread_id);
	chronos_mutex_unlock(r1);
	end_rtseg_self(END_PRIO);
	THREAD_OUTPUT("thread %d - end\n", thread_id);

	test_val -= 29; //88986112, 5.0 seconds

	pthread_exit(NULL);
}

void *task4(void *p)
{
	int thread_id = gettid();
	sched_setaffinity(0, sizeof(mask3), (cpu_set_t *) &mask3);
	THREAD_OUTPUT("thread %d - start\n", thread_id);

	test_val *= 19; //143317, 3.0 seconds

	clock_gettime(CLOCK_REALTIME, &deadline);
	deadline.tv_sec += 15;
	begin_rtseg_self(RUN_PRIO, 100, &deadline, NULL, 10000000);

	test_val *= 23; //3295808, 3.5 seconds

	//actual task block
	THREAD_OUTPUT("thread %d - burning cpu\n", thread_id);
	burn_cpu(4000000, exec_slope);

	end_rtseg_self(END_PRIO);
	THREAD_OUTPUT("thread %d - end\n", thread_id);

	test_val -= 3; //8898619, 6.5 seconds

	pthread_exit(NULL);
}

int main(int argc, char* argv[])
{
	pthread_t t1, t2, t3, t4;
	char c, optstring[]= "vh";
	int proc_count;

	r1 = new chronos_mutex_t();
	chronos_mutex_init(r1);
	sched = SCHED_RT_GGUA;

	while ((c = getopt (argc, argv, optstring)) != -1)
	{
		switch (c) {
		case 'v' :
			verbose = true;
			break;
		case 'h' :
			cout << "A simple test for priority inheritance behavior under global scheduling. Use -v for detailed output" << endl;
			return 0;
		}
	}

#ifdef _SC_NPROCESSORS_ONLN
		proc_count = sysconf(_SC_NPROCESSORS_ONLN);
#endif

	if(proc_count == 1) {
		cout << "Global scheduling test not applicable to single-core machine" << endl;
		return 0;
	}

	exec_slope = get_burn_cpu_slope();
	if(exec_slope == 0) {
		cout << "Error: No slope found" << endl;
		return 0;
	}

	deadline.tv_sec = 0;
	deadline.tv_nsec = 0;

	if(set_scheduler(sched, RUN_PRIO, mask3)) {
		cout << "Error: Unable to set scheduler" << endl;
		cout << "Are you running as root?" << endl;
	}

	if(prep_test())
		return 1;

	test_val +=5;

	pthread_create(&t1, NULL, task1, NULL);
	sleep(1);
	pthread_create(&t2, NULL, task2, NULL);
	sleep(1);
	pthread_create(&t3, NULL, task3, NULL);
	sleep(1);
	pthread_create(&t4, NULL, task4, NULL);
	pthread_join(t2, NULL);
	pthread_join(t1, NULL);
	pthread_join(t4, NULL);
	pthread_join(t3, NULL);

	if(test_val == 88986104)
		cout << "Global PI test successful" << endl;
	else
		cout << "Global PI test failed: " << test_val << endl;

	chronos_mutex_destroy(r1);
	delete r1;
	return 0;
}

