#include "test.h"

unsigned long mask1 = 1, mask3 = 3;
struct timespec period;

void *task1(void *p)
{
	int thread_id = gettid();
	sched_setaffinity(0, sizeof(mask3), (cpu_set_t *) &mask3);

	THREAD_OUTPUT("thread %d - start\n", thread_id);
	test_val *= 7; //56, 0.0 seconds
	period.tv_sec = 4;
	begin_rtseg_selfbasic(RUN_PRIO, NULL, &period);

	//actual task block - 3.0 seconds
	THREAD_OUTPUT("thread %d - burning cpu\n", thread_id);
	burn_cpu(200000, exec_slope);
	test_val *= 11; //517, 0.2 seconds
	burn_cpu(2800000, exec_slope);
	end_rtseg_self(END_PRIO);
	THREAD_OUTPUT("thread %d - end\n", thread_id);
	test_val -= 29; //88986112
	pthread_exit(NULL);
}

void *task2(void *p)
{
	int thread_id = gettid();
	sched_setaffinity(0, sizeof(mask3), (cpu_set_t *) &mask3);

	THREAD_OUTPUT("thread %d - start\n", thread_id);
	test_val -= 9; //47, 0.0 seconds
	period.tv_sec = 1;
	begin_rtseg_selfbasic(RUN_PRIO, NULL, &period);

	//actual task block - 2.0 seconds
	THREAD_OUTPUT("thread %d - burning cpu\n", thread_id);
	burn_cpu(400000, exec_slope);
	test_val -= 13; //504, 0.4 seconds
	burn_cpu(1600000, exec_slope);
	end_rtseg_self(END_PRIO);
	THREAD_OUTPUT("thread %d - end\n", thread_id);
	test_val -= 21; //143296, 2.0 seconds
	pthread_exit(NULL);
}

void *task3(void *p)
{
	int thread_id = gettid();
	sched_setaffinity(0, sizeof(mask3), (cpu_set_t *) &mask3);

	THREAD_OUTPUT("thread %d - start\n", thread_id);
	test_val *= 15; //7560, 1.0 seconds
	period.tv_sec = 3;
	begin_rtseg_selfbasic(RUN_PRIO, NULL, &period);

	//actual task block - 2.0 seconds
	THREAD_OUTPUT("thread %d - burning cpu\n", thread_id);
	burn_cpu(200000, exec_slope);
	test_val *= 23; //3295808, 2.2 seconds
	burn_cpu(1800000, exec_slope);
	end_rtseg_self(END_PRIO);
	THREAD_OUTPUT("thread %d - end\n", thread_id);
	test_val *= 27; //88986141, 4.0 seconds
	pthread_exit(NULL);
}

void *task4(void *p)
{
	int thread_id = gettid();
	sched_setaffinity(0, sizeof(mask3), (cpu_set_t *) &mask3);

	THREAD_OUTPUT("thread %d - start\n", thread_id);
	test_val -= 17; //7543, 1.0 seconds
	period.tv_sec = 2;
	begin_rtseg_selfbasic(RUN_PRIO, NULL, &period);

	//actual task block - 2.0 seconds
	THREAD_OUTPUT("thread %d - burning cpu\n", thread_id);
	burn_cpu(100000, exec_slope);
	test_val *= 19; //143317, 1.1 seconds
	burn_cpu(1900000, exec_slope);
	end_rtseg_self(END_PRIO);
	THREAD_OUTPUT("thread %d - end\n", thread_id);
	test_val -= 25; //3295783, 3.0 seconds
	pthread_exit(NULL);
}

int main(int argc, char* argv[])
{
	pthread_t t1, t2, t3, t4;
	int proc_count = 1;
	sched = SCHED_RT_GRMA;

	while ((c = getopt (argc, argv, optstring)) != -1)
	{
		switch (c) {
		case 'v' :
			verbose = true;
			break;
		case 'h' :
			printf("A simple test for proper stw global scheduling behavior. Use -v for detailed output\n");
			return 0;
		}
	}

#ifdef _SC_NPROCESSORS_ONLN
	proc_count = sysconf(_SC_NPROCESSORS_ONLN);
#endif

	if(proc_count == 1) {
		printf("Global scheduling test not applicable to single-core machine\n");
		return 0;
	}

	exec_slope = get_burn_cpu_slope();
	if(exec_slope == 0) {
		cout << "Error: No slope found" << endl;
		return 0;
	}

	if(prep_test())
		return 1;

	period.tv_sec = 0;
	period.tv_nsec = 0;

	set_scheduler(sched, RUN_PRIO, mask3);

	test_val +=5;

	/* 1,2 start, 3,4 start, 4 preempt 1, 2 finish, select 3, 4 finish select 1 */

	pthread_create(&t1, NULL, task1, NULL);
	pthread_create(&t2, NULL, task2, NULL);
	sleep(1);
	pthread_create(&t3, NULL, task3, NULL);
	pthread_create(&t4, NULL, task4, NULL);
	sleep(1);
	pthread_join(t2, NULL);
	pthread_join(t4, NULL);
	pthread_join(t1, NULL);
	pthread_join(t3, NULL);

	if(test_val == 88986112)
		printf("Global scheduling test (2/2) successful\n");
	else
		printf("Global scheduling test (2/2) failed: %d\n", test_val);

	return 0;
}




