/***************************************************************************
 *   Copyright (C) 2011 by Virginia Tech Real-Time Systems Lab             *
 *                                                                         *
 *   Author(s):                                                            *
 *       - Aaron Lindsay (aaron.lindsay@vt.edu)                            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

package edu.vt.realtime;

import java.nio.ByteBuffer;

public class ChronosScheduler {
	public ChronosScheduler() { }

	/* Make sure libjchronos is loaded so we can access the native methods */
	static{
		System.loadLibrary("jchronos");
	}

	private static final native boolean chronos_beginSegment(int tid, int priority, int maxutil, long deadline_sec, long deadline_ns, long period_sec, long period_ns, long exectime_us);

	public static final boolean beginSegment(int priority, Timespec deadline, Timespec period) {
		return ChronosScheduler.chronos_beginSegment(0, priority, 0, deadline.getSeconds(), deadline.getNanoseconds(), period.getSeconds(), period.getNanoseconds(), 0);
	}

	public static final boolean beginSegment(int tid, int priority, Timespec deadline, Timespec period) {
		return ChronosScheduler.chronos_beginSegment(tid, priority, 0, deadline.getSeconds(), deadline.getNanoseconds(), period.getSeconds(), period.getNanoseconds(), 0);
	}

	public static final boolean beginSegment(int priority, int maxutil, Timespec deadline, Timespec period, long exectime_us) {
		return ChronosScheduler.chronos_beginSegment(0, priority, maxutil, deadline.getSeconds(), deadline.getNanoseconds(), period.getSeconds(), period.getNanoseconds(), exectime_us);
	}

	public static final boolean beginSegment(int tid, int priority, int maxutil, Timespec deadline, Timespec period, long exectime_us) {
		return ChronosScheduler.chronos_beginSegment(tid, priority, maxutil, deadline.getSeconds(), deadline.getNanoseconds(), period.getSeconds(), period.getNanoseconds(), exectime_us);
	}

	private static final native boolean chronos_endSegment(int tid, int priority);

	public static final boolean endSegment(int priority) {
		return ChronosScheduler.chronos_endSegment(0, priority);
	}

	public static final boolean endSegment(int tid, int priority) {
		return ChronosScheduler.chronos_endSegment(tid, priority);
	}

	private static final native boolean chronos_setScheduler(long scheduler, int priority, long cpus);

	public static final boolean setScheduler(ChronosSchedulerType scheduler, int priority, long cpus) {
		return ChronosScheduler.chronos_setScheduler(scheduler.getNativeId(), priority, cpus);
	}

	private static final native ByteBuffer chronos_initAborts();

	public static final ChronosAborts initAborts() {
		ByteBuffer buff = ChronosScheduler.chronos_initAborts();
		if(buff != null)
			return new ChronosAborts(buff);
		return null;
	}

	private static native boolean chronos_addAbortHandler(int tid, int max_util, long tv_sec, long tv_nsec, long exec_time);

	//adds abort handler to calling thread
	public static boolean addAbortHandler(int max_util, Timespec deadline, long exec_time) {
		return ChronosScheduler.addAbortHandler(0, max_util, deadline, exec_time);
	}

	//adds abort handler to thread specified by tid
	public static boolean addAbortHandler(int tid, int max_util, Timespec deadline, long exec_time){
		return ChronosScheduler.chronos_addAbortHandler(tid, max_util, deadline.getSeconds(), deadline.getNanoseconds(), exec_time);
	}

	private static native boolean chronos_addAbortHandler_nodeadline(int tid, int max_util, long exec_time);

	//adds abort handler to calling thread (with no deadline)
	public static boolean addAbortHandler(int max_util, long exec_time) {
		return ChronosScheduler.addAbortHandler(0, max_util, exec_time);
	}

	//adds abort handler to thread specified by tid (with no deadline)
	public static boolean addAbortHandler(int tid, int max_util, long exec_time){
		return ChronosScheduler.chronos_addAbortHandler_nodeadline(tid, max_util, exec_time);
	}
}
