/***************************************************************************
 *   Copyright (C) 2011 by Virginia Tech Real-Time Systems Lab             *
 *                                                                         *
 *   Author(s):                                                            *
 *       - Aaron Lindsay (aaron.lindsay@vt.edu)                            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

package edu.vt.realtime;

public enum ChronosSchedulerType {
	SCHED_RT_FIFO(0x0000),
	SCHED_RT_RMA(0x0100),
	SCHED_RT_EDF(0x0200),
	SCHED_RT_HVDF(0x0300),
	SCHED_RT_LBESA(0x0400),
	SCHED_RT_DASA(0x0600),
	SCHED_RT_DASA_ND(0x0500),
	SCHED_RT_GFIFO(0x8000),
	SCHED_RT_GRMA(0x8100),
	SCHED_RT_GEDF(0x8200),
	SCHED_RT_GNP_EDF(0x8300),
	SCHED_RT_GHVDF(0x8400),
	SCHED_RT_GMUA(0x8500),
	SCHED_RT_GGUA(0x8600),
	SCHED_RT_NGGUA(0x8700),
	SCHED_RT_GNP_HVDF(0x8800);

	private final long nativeId;

	private ChronosSchedulerType(final long nativeId) {
		this.nativeId = nativeId; 
	}

	public final String displayString() {
		return this.name() + "(" + String.format("%h",this.nativeId) + ")";
	}

	public long getNativeId() {
		return nativeId;
	}

	public static ChronosSchedulerType fromString(String sched) {
		for(ChronosSchedulerType t : ChronosSchedulerType.values()) {
			String name = t.toString().substring(9);
			if(name.equalsIgnoreCase(sched))
				return t;
		}
		return null;
	}
}
