/***************************************************************************
 *   Copyright (C) 2011 by Virginia Tech Real-Time Systems Lab             *
 *                                                                         *
 *   Author(s):                                                            *
 *       - Aaron Lindsay (aaron.lindsay@vt.edu)                            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

package edu.vt.realtime;

public class Timespec{
	private long tv_sec;
	private long tv_nsec;
	
	public Timespec() {
		tv_sec = 0;
		tv_nsec = 0;
	}

	public Timespec(long seconds, long nsec) {
		tv_sec = seconds;
		tv_nsec = nsec;
		normalize();
	}

	public native void subtract(Timespec op);
	public native void add(Timespec op);
	
	/* Called to make sure the nsec value is in the desired range */
	private void normalize() {
		while(this.tv_nsec < 0) {
			this.tv_sec--;
			this.tv_nsec += 1000000000; //a billion, the conversion b/n seconds and nanoseconds
		}

		while(this.tv_nsec >= 1000000000) {
			this.tv_sec++;
			this.tv_nsec -= 1000000000; //a billion, the conversion b/n seconds and nanoseconds
		}
	}

	public long getSeconds() { return tv_sec; }
	public long getNanoseconds() { return tv_nsec; }
	public void setTime(long sec, long nsec) { setSeconds(sec); setNanoseconds(nsec); }
	public void setTime(Timespec op) { setSeconds(op.getSeconds()); setNanoseconds(op.getNanoseconds()); }
	public void setSeconds(long sec) { this.tv_sec = sec; }
	public void setNanoseconds(long nsec) { this.tv_nsec = nsec; this.normalize(); }

	public boolean lessThan(Timespec op) {
		return (this.tv_sec < op.tv_sec) || (this.tv_sec == op.tv_sec && this.tv_nsec < op.tv_nsec);
	}
	
	public boolean greaterThan(Timespec op) {
		return (this.tv_sec > op.tv_sec) || (this.tv_sec == op.tv_sec && this.tv_nsec > op.tv_nsec);
	}

	public boolean equals(Timespec op) {
		return this.tv_sec == op.tv_sec && this.tv_nsec == op.tv_nsec;
	}

	public Timespec copy() {
		return new Timespec(this.tv_sec, this.tv_nsec);
	}

	public long inNanoseconds() {
		return this.tv_sec*1000000000 + this.tv_nsec;
	}

	/* Calls clock_gettime in C */
	public native void getTime();

	public String toString() {
		return this.tv_sec + " sec + " + this.tv_nsec + " nsec";
	}
}
