/***************************************************************************
 *   Copyright (C) 2011 by Virginia Tech Real-Time Systems Lab             *
 *                                                                         *
 *   Author(s):                                                            *
 *       - Aaron Lindsay (aaron.lindsay@vt.edu)                            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <edu_vt_realtime_ChronosScheduler.h>

#include <chronos/chronos.h>
#include <chronos/chronos_aborts.h>
#include <time.h>

JNIEXPORT jboolean JNICALL Java_edu_vt_realtime_ChronosScheduler_chronos_1beginSegment (JNIEnv * env, jclass class, jint tid, jint priority, jint maxutil, jlong deadline_sec, jlong deadline_ns, jlong period_sec, jlong period_ns, jlong exectime_us) {
	struct timespec deadline, period;

	deadline.tv_sec=deadline_sec;
	deadline.tv_nsec=deadline_ns;
	period.tv_sec=period_sec;
	period.tv_nsec=period_ns;

	return begin_rtseg(tid, priority, maxutil, &deadline, &period, exectime_us)==0;
}

JNIEXPORT jboolean JNICALL Java_edu_vt_realtime_ChronosScheduler_chronos_1endSegment (JNIEnv * env, jclass class, jint tid, jint priority) {
	return end_rtseg(tid, priority)==0;
}

JNIEXPORT jboolean JNICALL Java_edu_vt_realtime_ChronosScheduler_chronos_1setScheduler (JNIEnv * env, jclass class, jlong scheduler, jint priority, jlong cpus) {
	unsigned long mask=(unsigned long)cpus;
	return set_scheduler(scheduler, priority, mask)==0;
}

JNIEXPORT jobject JNICALL Java_edu_vt_realtime_ChronosScheduler_chronos_1initAborts (JNIEnv * env, jclass class) {
	jobject buffer=NULL;
	chronos_aborts_t abort_data;

	/*Initialize the shared memory for aborts*/
	if(init_aborts(&abort_data) || !abort_data.initialized)
		return NULL; /*return NULL if it fails*/


	/* Create new java.nio.ByteBuffer */
	buffer = (*env)->NewDirectByteBuffer(env, abort_data.mmap_pointer, abort_data.mapsize);
	return buffer;
}

JNIEXPORT jboolean JNICALL Java_edu_vt_realtime_ChronosScheduler_chronos_1addAbortHandler (JNIEnv * env, jclass class, jint tid, jint max_util, jlong tv_sec, jlong tv_nsec, jlong exec_time){
	struct timespec deadline;
	deadline.tv_sec = tv_sec;
	deadline.tv_nsec = tv_nsec;
	return add_abort_handler(tid, max_util, &deadline, (unsigned long)exec_time) == 0;
}

JNIEXPORT jboolean JNICALL Java_edu_vt_realtime_ChronosScheduler_chronos_1addAbortHandler_1nodeadline (JNIEnv * env, jclass class, jint tid, jint max_util, jlong exec_time){
	return add_abort_handler_nodeadline(tid, max_util, (unsigned long)exec_time) == 0;
}
