
/***************************************************************************
 *   Copyright (C) 2011 by Virginia Tech Real-Time Systems Lab             *
 *                                                                         *
 *   Author(s):                                                            *
 *       - Aaron Lindsay (aaron.lindsay@vt.edu)                            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <edu_vt_realtime_ChronosMutex.h>

#include <chronos/chronos.h>

/* Convert a chronos_mutex_t* in C to a ChronosMutex in Java */
void cToJava(chronos_mutex_t * m, JNIEnv * env, jobject this) {
	jfieldID value_fid;	/* the IDs of the instance variables */
	jfieldID owner_fid;
	jfieldID id_fid;
	jclass cls = (*env)->GetObjectClass(env, this);	/* the 'C' class of ChronosMutex */
	value_fid = (*env)->GetFieldID(env, cls, "value", "I");
	owner_fid = (*env)->GetFieldID(env, cls, "owner", "I");
	id_fid = (*env)->GetFieldID(env, cls, "id", "J");

	if (!value_fid || !owner_fid || !id_fid) {
		//TODO raise an exception?!?
		return;
	}

	(*env)->SetIntField(env, this, value_fid, (jint)m->value);
	(*env)->SetIntField(env, this, owner_fid, (jint)m->owner);
	(*env)->SetLongField(env, this, id_fid, (jlong)m->id);
}

/* Convert a ChronosMutex from Java to a chronos_mutex_t* in C */
void javaToC(chronos_mutex_t * m, JNIEnv * env, jobject this) {
	jfieldID value_fid;	/* the IDs of the instance variables */
	jfieldID owner_fid;
	jfieldID id_fid;
	jclass cls = (*env)->GetObjectClass(env, this);	/* the 'C' class of ChronosMutex */
	value_fid = (*env)->GetFieldID(env, cls, "value", "I");
	owner_fid = (*env)->GetFieldID(env, cls, "owner", "I");
	id_fid = (*env)->GetFieldID(env, cls, "id", "J");

	if (!value_fid || !owner_fid || !id_fid) {
		//TODO raise an exception?!?
		return;
	}

	m->value = (u_int32_t)(*env)->GetIntField(env, this, value_fid);
	m->owner = (int)(*env)->GetIntField(env, this, owner_fid);
	m->id = (unsigned long)(*env)->GetLongField(env, this, id_fid);
}

JNIEXPORT void JNICALL Java_edu_vt_realtime_ChronosMutex_mutexInit (JNIEnv * env, jobject this) {
	chronos_mutex_t m;
	chronos_mutex_init(&m);
	cToJava(&m, env, this);
}

JNIEXPORT void JNICALL Java_edu_vt_realtime_ChronosMutex_mutexDestroy (JNIEnv * env, jobject this) {
	chronos_mutex_t m;
	javaToC(&m, env, this);
	chronos_mutex_destroy(&m);
}

JNIEXPORT jboolean JNICALL Java_edu_vt_realtime_ChronosMutex_mutexLock (JNIEnv * env, jobject this) {
	jboolean ret;
	chronos_mutex_t m;
	javaToC(&m, env, this);
	ret = chronos_mutex_lock(&m) > -1;
	cToJava(&m, env, this);
	return ret;
}

JNIEXPORT jboolean JNICALL Java_edu_vt_realtime_ChronosMutex_mutexUnlock (JNIEnv * env, jobject this) {
	jboolean ret;
	chronos_mutex_t m;
	javaToC(&m, env, this);
	ret = !chronos_mutex_unlock(&m);
	cToJava(&m, env, this);
	return ret;
}
