#include "edu_vt_realtime_Timespec.h"

#include <time.h>

void timespec_cToJava(JNIEnv * env, jobject this, struct timespec * t) {
	jfieldID sec_fid;
	jfieldID nsec_fid;
	jclass cls = (*env)->GetObjectClass(env, this);	/* the 'C' class of Timespec */
	/* get the fid's of the instance variables */
	sec_fid = (*env)->GetFieldID(env, cls, "tv_sec", "J");
	nsec_fid = (*env)->GetFieldID(env, cls, "tv_nsec", "J");

	/* Set the time in the Java Timespec object */
	(*env)->SetLongField(env, this, sec_fid, t->tv_sec);
	(*env)->SetLongField(env, this, nsec_fid, t->tv_nsec);
}

void timespec_javaToC(JNIEnv * env, jobject this, struct timespec * t) {
	jfieldID sec_fid;
	jfieldID nsec_fid;
	jclass cls = (*env)->GetObjectClass(env, this);	/* the 'C' class of Timespec */
	/* get the fid's of the instance variables */
	sec_fid = (*env)->GetFieldID(env, cls, "tv_sec", "J");
	nsec_fid = (*env)->GetFieldID(env, cls, "tv_nsec", "J");

	t->tv_sec = (*env)->GetLongField(env, this, sec_fid);
	t->tv_nsec = (*env)->GetLongField(env, this, nsec_fid);
}

/* Add o to t, and store the result in t */
inline void add(struct timespec * t, long long tv_sec, long long tv_nsec) {
	long long nsec;

	nsec = t->tv_nsec + tv_nsec;
	t->tv_sec += tv_sec + (unsigned long)(nsec / 1000000000);
	t->tv_nsec = (unsigned long)(nsec % 1000000000);
}

JNIEXPORT void JNICALL Java_edu_vt_realtime_Timespec_subtract (JNIEnv * env, jobject this, jobject op) {
	struct timespec t, o;
	timespec_javaToC(env, this, &t);
	timespec_javaToC(env, op, &o);

	add(&t, -o.tv_sec, -o.tv_nsec);

	timespec_cToJava(env, this, &t);
}

JNIEXPORT void JNICALL Java_edu_vt_realtime_Timespec_add (JNIEnv * env, jobject this, jobject op) {
	struct timespec t, o;
	timespec_javaToC(env, this, &t);
	timespec_javaToC(env, op, &o);

	add(&t, o.tv_sec, o.tv_nsec);

	timespec_cToJava(env, this, &t);
}

JNIEXPORT void JNICALL Java_edu_vt_realtime_Timespec_getTime (JNIEnv * env, jobject this){
	struct timespec t;

	/* get the time from the realtime clock */
	clock_gettime(CLOCK_REALTIME, &t);

	timespec_cToJava(env, this, &t);
}
