/***************************************************************************
 *   Copyright (C) 2011 by Virginia Tech Real-Time Systems Lab             *
 *                                                                         *
 *   Author(s):                                                            *
 *       - Aaron Lindsay (aaron.lindsay@vt.edu)                            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "edu_vt_realtime_ChronosUtil.h"

#include <chronos/chronos_utils.h>
#include <sched.h>
#include <sys/time.h>
#include <sys/resource.h>
#include <sys/mman.h>
#include <errno.h>
#include <unistd.h>

JNIEXPORT void JNICALL Java_edu_vt_realtime_ChronosUtil_burnCpu (JNIEnv * env, jclass class, jlong exec_usec, jdouble slope) {
	burn_cpu((long long)exec_usec, (double)slope);
}

JNIEXPORT void JNICALL Java_edu_vt_realtime_ChronosUtil_usleep (JNIEnv * env, jclass class, jlong usec) {
	usleep(usec);
}

JNIEXPORT void JNICALL Java_edu_vt_realtime_ChronosUtil_printAbortTime (JNIEnv * env, jclass class, jint tid){
	long long end_time = RDTSC();
	printf("%d %lld\n", tid, end_time);
}

JNIEXPORT jint JNICALL Java_edu_vt_realtime_ChronosUtil_gettid (JNIEnv * env, jclass class) {
	return gettid();
}

JNIEXPORT jlong JNICALL Java_edu_vt_realtime_ChronosUtil_gcd (JNIEnv * env, jclass class, jlong m, jlong n) {
	return (jlong)gcd((unsigned long)m, (unsigned long)n);
}

JNIEXPORT jboolean JNICALL Java_edu_vt_realtime_ChronosUtil_setAffinity (JNIEnv * env, jclass class, jint pid, jlong mask) {
	unsigned long realmask=(unsigned long)mask;
	unsigned int len = sizeof(realmask);

	return !sched_setaffinity((pid_t)pid, len, (cpu_set_t *) &realmask);
}

JNIEXPORT jint JNICALL Java_edu_vt_realtime_ChronosUtil_getPriority (JNIEnv * env, jclass class) {
	int prio;
	errno = 0;
	prio = getpriority(PRIO_PROCESS, 0);
	if (prio == -1 && errno < 0)
		return -100;
	return prio;
}

JNIEXPORT jboolean JNICALL Java_edu_vt_realtime_ChronosUtil_setPriority (JNIEnv * env, jclass class, jint priority) {
	return !setpriority(PRIO_PROCESS, 0, (int)priority);
}

JNIEXPORT int JNICALL Java_edu_vt_realtime_ChronosUtil_1getLinuxScheduler (JNIEnv * env, jclass class) {
	return sched_getscheduler(0);
}

JNIEXPORT jboolean JNICALL Java_edu_vt_realtime_ChronosUtil_setLinuxScheduler (JNIEnv * env, jclass class, jint scheduler, jint priority) {
	struct sched_param param;
	param.sched_priority = priority;
	return !sched_setscheduler(0, (int)scheduler, &param);
}

JNIEXPORT jboolean JNICALL Java_edu_vt_realtime_ChronosUtil_memoryLock (JNIEnv * env, jclass class) {
	return !mlockall(MCL_CURRENT|MCL_FUTURE);
}

JNIEXPORT jboolean JNICALL Java_edu_vt_realtime_ChronosUtil_memoryUnlock (JNIEnv * env, jclass class) {
	return !munlockall();
}
