/***************************************************************************
 *   Copyright (C) 2009-2011 Virginia Tech Real-Time Systems Lab           *
 *                                                                         *
 *   Written by Aaron Lindsay                                              *
 *   aaron@aclindsay.com                                                   *
 *                                                                         *
 *   Based on code written by Matthew Dellinger                            *
 *   mdelling@vt.edu                                                       *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <sys/mman.h>

#include "../utils.h"
#include "../workload.h"
#include "slope.h"

void print_usage()
{
	printf("Slope generation application for ChronOS Linux (version ");
	printf(VERSION);
	printf(")\n");
	printf("------------------------------------------------------\n");
	printf("Optional flags:\n");
	printf
	    ("  -f            Force generation of slopes that already exist.\n");
	printf
	    ("  -v            Verbose output (show each individual slope sample as it is calculated, among other things).\n");
	printf("\n");
}

int main(int argc, char *argv[])
{
int c;
char  optstring[] = "fv";
	int force = 0;		//force regeneration of pre-existing slopes
	int verbose = 0;
	int i;

	while ((c = getopt(argc, argv, optstring)) != -1) {
		switch (c) {
		case 'f':
			force = 1;
			break;
		case 'v':
			verbose = 1;
			break;
		default:
			print_usage();
			return 0;
		}
	}

	//attempt to lock the memory of this process
	if (mlockall(MCL_CURRENT | MCL_FUTURE)) {
		printf("Error: Unable to lock the memory space.\n");
		printf("Make sure you are running with sudo or root.\n");
		return 1;
	}

	if (force)
		printf("Updating all slopes. ");
	else
		printf
		    ("Updating any slopes which have not previously been calculated. ");
	printf
	    ("This process may take a minute or two for each slope value calculated. \n\n");

	//loop through all workloads, finding the slope for each as we go
	for (i = 0; i < NUM_WORKLOADS; i++)
		update_slope(i, force, verbose);

	return 0;
}
