/***************************************************************************
 *   Copyright (C) 2009-2011 Virginia Tech Real-Time Systems Lab           *
 *                                                                         *
 *   Written by Aaron Lindsay                                              *
 *   aaron@aclindsay.com                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "../workload_type.h"

#ifndef ARRAY_WALK_H
#define ARRAY_WALK_H

#define ARRAY_SIZE 64		//kilobytes
#define STRIDE 64/sizeof(int)	//bytes

struct array_walk_data {
	int *array;
	long num_elements;
	long next_element;
};

static void *array_walk_init_task()
{
	struct array_walk_data *data = malloc(sizeof(struct array_walk_data));
	assert(data);
	data->next_element = 0;
	data->num_elements = (ARRAY_SIZE * 1024) / sizeof(int);
	data->array = (int *)malloc(sizeof(int) * data->num_elements);
	assert(data->array);
	return data;
}

static void array_walk_cleanup_task(void *ptr)
{
	struct array_walk_data *data = (struct array_walk_data *)ptr;
	free(data->array);
	data->array = 0;
	free(data);
}

static void array_walk_do_work(void *ptr, unsigned long iterations)
{
	unsigned long i;
	struct array_walk_data *data = (struct array_walk_data *)ptr;
	for (i = 0; i < iterations; i++) {
		data->array[data->next_element]++;
//              data->next_element = (data->next_element + STRIDE) % data->num_elements; //stride forwards STRIDE elements
		data->next_element = (data->num_elements + data->next_element - STRIDE) % data->num_elements;	//stride backwards STRIDE elements
	}
}

static const char array_walk_name[] = "array_walk";
static struct workload array_walk_workload = {
	.name = array_walk_name,
	.init_global = 0,
	.init_task = array_walk_init_task,
	.cleanup_global = 0,
	.cleanup_task = array_walk_cleanup_task,
	.do_work = array_walk_do_work
};

#endif				/* ARRAY_WALK_H */
